%{
#include <math.h>	/* for atof() */
#include <netdb.h>	/* for gethostbyname */
#include "dq.h"
#include "parser.tab.h"
#include <stdlib.h>	/* for exit */

#define YY_INPUT(buf,result,max_size) {result = readit(buf, max_size);}

#define YY_NO_INPUT     /* avoid a compiler warning */

extern int error_already;

int dohost(char* text){
    struct hostent* myhost;
    myhost = gethostbyname(text);
    if (!myhost) {
      fprintf(stderr, "Host '%s' not found.\n", text);
      yylval = 0;
      error_already = 1;
      return OOPS;
    }
    yylval = ntohl(*((unsigned int*)(myhost-> h_addr)));
    return Vnum;
}

%}

DIGIT	[0-9]
NUM	{DIGIT}+
NORMAL	[a-z][a-z0-9]*
WEB	[a-zA-Z0-9][-a-zA-Z0-9]*
SUFFIX	[.]{WEB}
HOST	{WEB}{SUFFIX}*[.]?
LCWEB	[a-z0-9][-a-z0-9]*
LCSUFFIX [.]{LCWEB}
LCHOST	{LCWEB}{LCSUFFIX}*[.]?

%%

{NUM}	 {
  yylval = atoi(yytext);
  return Snum;
}

{NUM}\.{NUM}\.{NUM}\.{NUM}	   {
  int a,b,c,d;
  sscanf(yytext, "%d.%d.%d.%d", &a, &b, &c, &d);
  yylval = (a<<24) + (b<<16) + (c<<8) + d;
  return Vnum;
}

Scalar|SCALAR {
  return SCALAR;
}

Vector|VECTOR {
  return VECTOR;
}

Lsh|LSH|"<<" {
  return LSH;
}

Rsh|RSH|">>" {
  return RSH;
}

Hi|HI|">>" {
  return HI;
}

Lo|LO|">>" {
  return LO;
}

Div|DIV {
  return '/';
}

Mul|MUL {
  return '*';		// useful because "*" is meaningful to the shell
}

And|AND {
  return '&';
}

Or|OR {
  return '|';
}

Xor|XOR {
  return '^';
}

Not|NOT {
  return '~';
}

Card|CARD {
  return CARD;
}

Swap|SWAP {
  return SWAP;
}

Last|LAST {
  return LAST;
}

H|Help|HELP {
  return HELP;
}


"+"|"-"|"*"|"/"|"%"|"^"|"("|")"|"|"|"&"|"!"|"~"|\n|";"|","   {
	  return yytext[0];	// single-character tokens
}

{LCHOST} {
  return dohost(yytext);
}

{HOST} {
  fprintf(stderr, "Warning: '%s' is not a keyword; "
  	" hostnames should be lowercase.\n", yytext);
  return dohost(yytext);
}

"#"[^\n]*	// eat up one-line comments


[ \t]+		/* eat up whitespace */

.	    printf( "Unrecognized character: %s\n", yytext );

%%

void junk(){
  yyunput(0,0);		// avoid strange link-time warning
}

int yywrap(){		// I don't know what this is supposed to do
  return 1;
}

#ifdef testmain

int main( int argc,  char** argv ){
   ++argv, --argc;  /* skip over program name */
   if ( argc > 0 ) yyin = fopen( argv[0], "r" );
   else yyin = stdin;
   yylex();
   return 0;
}

#endif
